@@ -12,6 +12,7 @@ mod tls;
 mod unix;
 
 use std::{
+    convert::Infallible,
     io::Error,
     pin::Pin,
     task::{Context, Poll},
@@ -153,6 +154,15 @@ pub trait Listener: Send {
     }
 }
 
+#[async_trait::async_trait]
+impl Listener for Infallible {
+    type Acceptor = Infallible;
+
+    async fn into_acceptor(self) -> IoResult<Self::Acceptor> {
+        unreachable!()
+    }
+}
+
 #[async_trait::async_trait]
 impl<T: Acceptor + ?Sized> Acceptor for Box<T> {
     type Io = T::Io;
@@ -166,6 +176,19 @@ impl<T: Acceptor + ?Sized> Acceptor for Box<T> {
     }
 }
 
+#[async_trait::async_trait]
+impl Acceptor for Infallible {
+    type Io = BoxIo;
+
+    fn local_addr(&self) -> Vec<LocalAddr> {
+        vec![]
+    }
+
+    async fn accept(&mut self) -> IoResult<(Self::Io, LocalAddr, RemoteAddr)> {
+        unreachable!()
+    }
+}
+
 /// An IO type for BoxAcceptor.
 pub struct BoxIo {
     reader: Box<dyn AsyncRead + Send + Unpin + 'static>,
